==> babel-plugin-styled-components-px2vw-1.1.2.tgz: package/src/index.ts <==
/*
 * Desc:
 * File: /src/index.ts
 * Project: node-typescript-template
 * File Created: Saturday, 3rd April 2021 1:26:29 pm
 * Author: luxuemin2108@gmail.com
 * -----
 * Copyright (c) 2021 Camel Lu
 */
import { ConfigAPI } from '@babel/core';
import { declare } from '@babel/helper-plugin-utils';
import { Visitor, NodePath } from '@babel/traverse';
import templateBuild from '@babel/template';
import { px2vw } from './px2vw';
import {
  Program,
  TaggedTemplateExpression,
  CallExpression,
  isCallExpression,
  TemplateElement,
  TemplateLiteral,
  // isTemplateLiteral,
  Identifier,
  isIdentifier,
  Expression,
  // isExpression,
  isTemplateElement,
  MemberExpression,
  isMemberExpression,
  isBlock,
  callExpression,
  arrowFunctionExpression,
  isArrowFunctionExpression,
  identifier,
  numericLiteral,
} from '@babel/types';
import configuration, { IConfiguration } from './configuration';
import { replace } from './replace';

function isStyledTagged(tagged: TaggedTemplateExpression) {
  const tag = tagged.tag;
  if (isIdentifier(tag)) {
    return isStyled(tag);
  } else if (isMemberExpression(tag) && isIdentifier(tag.object)) {
    return isStyled(tag.object);
  } else if (isCallExpression(tag)) {
    if (isIdentifier(tag.callee)) {
      return isStyled(tag.callee);
    } else if (isMemberExpression(tag.callee)) {
      return isStyledMember(tag.callee);
    }
  }
  return false;
}

function isStyledMember(member: MemberExpression): boolean {
  if (isIdentifier(member.object)) {
    return isStyled(member.object);
  } else if (isMemberExpression(member.object)) {
    return isStyledMember(member.object);
  }
  return false;
}

function isStyled(id: Identifier): boolean {
  return configuration.config.tags.indexOf(id.name) >= 0;
}

function transformTemplateElement(it: TemplateElement): void {
  if (it.value && it.value.raw) {
    it.value.raw = replace(it.value.raw);
  }
  if (it.value && it.value.cooked) {
    it.value.cooked = replace(it.value.cooked);
  }
}

let _px2vw: Identifier | undefined;

function transform(template: TemplateLiteral) {
  if (!template.expressions || template.expressions.length === 0) {
    template.quasis.forEach(transformTemplateElement);
  } else {
    const expressions: (Expression | TemplateElement)[] = [
      ...(template.expressions as Expression[]),
      ...template.quasis,
    ];
    expressions.sort((it1, it2) => (it1.start || 0) - (it2.start || 0));
    for (let i = 0; i < expressions.length; i++) {
      const expression = expressions[i];
      if (isTemplateElement(expression)) {
        transformTemplateElement(expression);
      } else if (_px2vw) {
        const next = expressions[i + 1];
        if (next && isTemplateElement(next)) {
          const text = next.value?.raw || next.value?.cooked;
          if (text && /^px/.test(text)) {
            if (isArrowFunctionExpression(expression)) {
              if (isBlock(expression.body)) {
                expression.body = callExpression(_px2vw, [arrowFunctionExpression([], expression.body)]);
              } else {
                expression.body = callExpression(_px2vw, [expression.body]);
              }
            } else {
              const idx = template.expressions.findIndex((it) => it === expression);
              if (idx !== -1) {
                template.expressions[idx] = callExpression(_px2vw, [expression]);
              }
            }
            if (next.value && next.value.raw) {
              next.value.raw = next.value.raw.replace(/^px/, '');
            }
            if (next.value && next.value.cooked) {
              next.value.cooked = next.value.cooked.replace(/^px/, '');
            }
          }
        }
      }
    }
  }
}

export default declare((api: ConfigAPI, options?: IConfiguration) => {
  api.assertVersion(7);
  configuration.updateConfig(options);
  const templateVisitor: Visitor = {
    TemplateLiteral: {
      // exit(templateLiteralPath: NodePath<TemplateLiteral>) {},
      enter(templateLiteralPath: NodePath<TemplateLiteral>) {
        transform(templateLiteralPath.node);
      },
    },
  };
  const visitor: Visitor = {
    Program: {
      exit() {
        _px2vw = undefined;
      },
      enter(programPath: NodePath<Program>) {
        if (configuration.config.transformRuntime) {
          _px2vw = programPath.scope.generateUidIdentifier('px2vw');
          const template = templateBuild.statement(px2vw);
          programPath.node.body.push(
            template({
              input: identifier('input'),
              px2vw: _px2vw,
              viewportWidth: numericLiteral(configuration.config.viewportWidth),
              unitPrecision: numericLiteral(configuration.config.unitPrecision),
              minPixelValue: numericLiteral(configuration.config.minPixelValue),
            }),
          );
        }
        programPath.traverse({
          TaggedTemplateExpression(path: NodePath<TaggedTemplateExpression>) {
            if (isStyledTagged(path.node)) {
              path.traverse(templateVisitor);
            }
          },
          CallExpression(path: NodePath<CallExpression>) {
            if (
              isMemberExpression(path.node.callee) &&
              isIdentifier(path.node.callee.object) &&
              isStyled(path.node.callee.object)
            ) {
              path.traverse(templateVisitor);
            }
          },
        });
      },
    },
  };

  return {
    name: 'styled-components-px2vw',
    visitor,
  };
});

==> babel-plugin-styled-components-px2vw-1.3.0.tgz: package/src/index.ts <==
/*
 * Desc:
 * File: /src/index.ts
 * Project: node-typescript-template
 * File Created: Saturday, 3rd April 2021 1:26:29 pm
 * Author: luxuemin2108@gmail.com
 * -----
 * Copyright (c) 2021 Camel Lu
 */
import { ConfigAPI } from '@babel/core';
import { declare } from '@babel/helper-plugin-utils';
import { Visitor, NodePath } from '@babel/traverse';
import templateBuild from '@babel/template';
import { px2vw } from './px2vw';
import {
  Program,
  TaggedTemplateExpression,
  CallExpression,
  isCallExpression,
  TemplateElement,
  TemplateLiteral,
  // isTemplateLiteral,
  Identifier,
  isIdentifier,
  Expression,
  // isExpression,
  isTemplateElement,
  MemberExpression,
  isMemberExpression,
  isBlock,
  callExpression,
  isConditionalExpression,
  arrowFunctionExpression,
  isArrowFunctionExpression,
  identifier,
  numericLiteral,
} from '@babel/types';
import configuration, { IConfiguration } from './configuration';
import { replace } from './replace';

function isStyledTagged(tagged: TaggedTemplateExpression) {
  const tag = tagged.tag;
  if (isIdentifier(tag)) {
    return isStyled(tag);
  } else if (isMemberExpression(tag) && isIdentifier(tag.object)) {
    return isStyled(tag.object);
  } else if (isCallExpression(tag)) {
    if (isIdentifier(tag.callee)) {
      return isStyled(tag.callee);
    } else if (isMemberExpression(tag.callee)) {
      return isStyledMember(tag.callee);
    }
  }
  return false;
}

function isStyledMember(member: MemberExpression): boolean {
  if (isIdentifier(member.object)) {
    return isStyled(member.object);
  } else if (isMemberExpression(member.object)) {
    return isStyledMember(member.object);
  }
  return false;
}

function isStyled(id: Identifier): boolean {
  return configuration.config.tags.indexOf(id.name) >= 0;
}

function transformTemplateElement(it: TemplateElement): void {
  if (it.value && it.value.raw) {
    it.value.raw = replace(it.value.raw);
  }
  if (it.value && it.value.cooked) {
    it.value.cooked = replace(it.value.cooked);
  }
}

function transformTemplateExpression(expression: Expression, px2vw: Identifier): Expression {
  if (isArrowFunctionExpression(expression)) {
    if (isBlock(expression.body)) {
      expression.body = callExpression(px2vw, [arrowFunctionExpression([], expression.body)]);
    } else {
      expression.body = callExpression(px2vw, [expression.body]);
    }
  } else if (isConditionalExpression(expression)) {
    expression.consequent = transformTemplateExpression(expression.consequent, px2vw);
    expression.alternate = transformTemplateExpression(expression.alternate, px2vw);
  } else {
    return callExpression(px2vw, [expression]);
  }

  return expression;
}

let _px2vw: Identifier | undefined;

function transform(template: TemplateLiteral) {
  if (!template.expressions || template.expressions.length === 0) {
    template.quasis.forEach(transformTemplateElement);
  } else {
    const expressions: (Expression | TemplateElement)[] = [
      ...(template.expressions as Expression[]),
      ...template.quasis,
    ];
    expressions.sort((it1, it2) => (it1.start || 0) - (it2.start || 0));
    for (let i = 0; i < expressions.length; i++) {
      const expression = expressions[i];
      if (isTemplateElement(expression)) {
        transformTemplateElement(expression);
      } else if (_px2vw) {
        const next = expressions[i + 1];
        if (next && isTemplateElement(next)) {
          const text = next.value?.raw || next.value?.cooked;
          if (text && /^px/.test(text)) {
            const idx = template.expressions.findIndex((it) => it === expression);
            if (idx !== -1) {
              template.expressions[idx] = transformTemplateExpression(expression, _px2vw);
            }
            if (next.value && next.value.raw) {
              next.value.raw = next.value.raw.replace(/^px/, '');
            }
            if (next.value && next.value.cooked) {
              next.value.cooked = next.value.cooked.replace(/^px/, '');
            }
          }
        }
      }
    }
  }
}

export default declare((api: ConfigAPI, options?: IConfiguration) => {
  api.assertVersion(7);
  configuration.updateConfig(options);
  const templateVisitor: Visitor = {
    TemplateLiteral: {
      // exit(templateLiteralPath: NodePath<TemplateLiteral>) {},
      enter(templateLiteralPath: NodePath<TemplateLiteral>) {
        transform(templateLiteralPath.node);
      },
    },
  };
  const visitor: Visitor = {
    Program: {
      exit() {
        _px2vw = undefined;
      },
      enter(programPath: NodePath<Program>) {
        if (configuration.config.transformRuntime) {
          _px2vw = programPath.scope.generateUidIdentifier('px2vw');
          const template = templateBuild.statement(px2vw);
          programPath.node.body.push(
            template({
              input: identifier('input'),
              px2vw: _px2vw,
              viewportWidth: numericLiteral(configuration.config.viewportWidth),
              unitPrecision: numericLiteral(configuration.config.unitPrecision),
              minPixelValue: numericLiteral(configuration.config.minPixelValue),
            }),
          );
        }
        programPath.traverse({
          TaggedTemplateExpression(path: NodePath<TaggedTemplateExpression>) {
            if (isStyledTagged(path.node)) {
              path.traverse(templateVisitor);
            }
          },
          CallExpression(path: NodePath<CallExpression>) {
            if (
              isMemberExpression(path.node.callee) &&
              isIdentifier(path.node.callee.object) &&
              isStyled(path.node.callee.object)
            ) {
              path.traverse(templateVisitor);
            }
          },
        });
      },
    },
  };

  return {
    name: 'styled-components-px2vw',
    visitor,
  };
});
